/* Generate random numbers to decide which route to enable */
int random_ab = Utils::randomInt(0, 2);
int random_ch = 1; // NOTE route C not yet implemented
int random_fi = Utils::randomInt(0, 2);
int random_iy = Utils::randomInt(0, 3); // This intersection is biased towards i
int random_px = Utils::randomInt(0, 3); // This intersection is biased towards p

/* Use these to force a route as enabled (true) or disabled (false) */
bool override_enabled = false;
bool override_0a = true;
bool override_0b = false;
bool override_ac = false;
bool override_bc = false;
bool override_ah = true;
bool override_bh = false;
bool override_hf = false;
bool override_hi = false;
bool override_ii = false;
bool override_io = false;
bool override_iy = false;
bool override_op = false;
bool override_ox = false;
bool override_s0 = false;

/* This SuperTuxKart map makes each route object call one of these
 *  functions to check whether enable that object or not.
 *
 * If either 'this route is randomly chosen' or 'we override this route',
 *  then return 'true', telling SuperTuxKart to enable this route.
 *
 * Function calls in later ones such as "route_a0(null)" are to check if
 *  a pre-requisite route has returned true, to prevent impossible tracks.
 */

/* Intersection 0-AB */

bool route_0a(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_0a;
    } else {
        return (random_ab == 0);
    }
}

bool route_0b(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_0b;
    } else {
        return (random_ab == 1);
    }
}

/* Intersection AB-CH */

bool route_ac(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_ac;
    } else {
        return (route_0a(null) && random_ch == 0);
    }
}

bool route_ah(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_ah;
    } else {
        return (route_0a(null) && random_ch == 1);
    }
}

bool route_bc(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_bc;
    } else {
        return (route_0b(null) && random_ch == 0);
    }
}

bool route_bh(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_bh;
    } else {
        return (route_0b(null) && random_ch == 1);
    }
}

/* Intersection H-FI */

bool route_hf(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_hf;
    } else {
        return ((route_ah(null) || route_bh(null)) && random_fi == 0);
    }
}

bool route_hi(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_hi;
    } else {
        return ((route_ah(null) || route_bh(null)) && random_fi == 1);
    }
}

/* Intersection I-IY */
// This intersection is biased toward i

bool route_ii(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_io;
    } else {
        return (route_hi(null) && random_iy < 2);
    }
}

bool route_iy(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_iy;
    } else {
        return (route_hi(null) && random_iy == 2);
    }
}

// Currently, route ii implies route io
bool route_io(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_io;
    } else {
        return route_ii(null);
    }
}

/* Intersection O-PX */
// This intersection is biased toward p

bool route_op(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_op;
    } else {
        return ((route_io(null) || route_hf(null)) && random_px < 2);
    }
}

bool route_ox(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_ox;
    } else {
        return ((route_io(null) || route_hf(null)) && random_px == 2);
    }
}

/* Straight S0 */
// This route has multiple entrances

bool route_s0(Track::TrackObject@ obj){
    if (override_enabled) {
        return override_s0;
    } else {
        return ((route_iy(null) || route_ox(null)));
    }
}